@@ -29,7 +29,9 @@ main ()
 {
         NotifyNotification *n;
 
+#if !GLIB_CHECK_VERSION (2, 36, 0)
         g_type_init ();
+#endif
 
         notify_init ("Error Handling");
 
