@@ -21,6 +21,7 @@
 #include "config.h"
 
 #include <libnotify/notify.h>
+#include <locale.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -125,6 +126,7 @@ main (int argc, char *argv[])
         static const char  *summary = NULL;
         char               *body;
         static const char  *type = NULL;
+        static char        *app_name = NULL;
         static char        *icon_str = NULL;
         static char        *icons = NULL;
         static char       **n_text = NULL;
@@ -146,6 +148,8 @@ main (int argc, char *argv[])
                  N_
                  ("Specifies the timeout in milliseconds at which to expire the "
                   "notification."), N_("TIME")},
+                {"app-name", 'a', 0, G_OPTION_ARG_STRING, &app_name,
+                 N_("Specifies the app name for the icon"), N_("APP_NAME")},
                 {"icon", 'i', 0, G_OPTION_ARG_FILENAME, &icons,
                  N_("Specifies an icon filename or stock icon to display."),
                  N_("ICON[,ICON...]")},
@@ -167,7 +171,11 @@ main (int argc, char *argv[])
 
         body = NULL;
 
+        setlocale (LC_ALL, "");
+
+#if !GLIB_CHECK_VERSION (2, 36, 0)
         g_type_init ();
+#endif
 
         g_set_prgname (argv[0]);
         g_log_set_always_fatal (G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
@@ -226,6 +234,7 @@ main (int argc, char *argv[])
         notify_notification_set_category (notify, type);
         notify_notification_set_urgency (notify, urgency);
         notify_notification_set_timeout (notify, expire_timeout);
+        notify_notification_set_app_name (notify, app_name);
 
         g_free (body);
 
@@ -237,7 +246,7 @@ main (int argc, char *argv[])
                 char         **tokens = NULL;
 
                 while ((hint = hints[i++])) {
-                        tokens = g_strsplit (hint, ":", -1);
+                        tokens = g_strsplit (hint, ":", 3);
                         l = g_strv_length (tokens);
 
                         if (l != 3) {
